<?php
/**
 * Plugin Name: Posts & Pages Changer
 * Plugin URI: https://plugins.svn.wordpress.org/posts-pages-changer
 * Description: Update posts and pages with help of changer.
 * Version: 1.0
 * Author: Janis Itkacs
 * Author URI: http://lapuvieta.lv
 * License: GPL2
 */


if (!defined( 'ABSPATH')) {
	exit;
} // Exit if accessed directly

if (!is_admin()) {
	return false;
} // Don't run if not admin page

class ji_wp_post_dater {

	public $plugin_title = "Posts & Pages Changer";
	public $abs_max_posts = 9999; // to not slow down the site! 
	public $earliest;

	function __construct() {

		$this->earliest_day = 5;
		$this->latest_day = 20;
		$this->post_type = array('post', 'page');
		$this->days_offset = null;
		$this->disbale_modified_days = null;

	}

	protected function get_all_posts() {

		$argsQueryPosts = array(
			'post_type' => $this->post_type,
			'posts_per_page' => -1,
		);

		// very quickly work out if we are sorting by cat  or tag
		$cat = '';
		if ($_POST['incategory'] != '') {
			$cat = '&cat=' . (int)$_POST['incategory'];
			$argsQueryPosts['cat'] = (int)$_POST['incategory'];
		}

		$tag = '';
		if ($_POST['tagid'] != '') {
			$tag = "&tag_id=" . (int)$_POST['tagid'];
			$argsQueryPosts['tag'] = (int)$_POST['tagid'];
		}

			$all_posts = array();
			$all_query = new WP_Query($argsQueryPosts);

			while($all_query->have_posts()) {
				$all_query->the_post();
				$orig_date = get_the_date("Y-m-d H:i:s");
				//$i++; unused variable
				$all_posts[get_the_ID()] = array
					(
						"link" => get_permalink(), 
						"title" => the_title('','',false),
						"orig_date" => $orig_date,
					);
			}

			return $all_posts;

	}

	public function redate_posts() {


		if (isset($_POST['latest_day']) && is_numeric($_POST['latest_day'])) {
			$this->latest_day = (int) $_POST['latest_day'];
		}

		if (isset($_POST['earliest_day']) && is_numeric($_POST['earliest_day'])) {
			$this->days_offset = (int) $_POST['earliest_day'];
		}

		if ($this->days_offset  <= 0) {
			$this->earliest_day = abs($this->days_offset);
		}

		if ($this->days_offset  > 0) {
			$this->latest_day = $this->days_offset;
			$this->earliest_day = 0;
		}

		if ($_POST['tagid'] !='' && $_POST['incategory'] != '') {
			echo "<p>Please go back - you cannot set both slug and category!</p>";
			return;
		}

		if (isset($_POST['earliest_modified_day']) && is_numeric($_POST['earliest_modified_day'])) {
			$this->earliest_modified_day = (int) $_POST['earliest_modified_day'];
		}

		if (isset($_POST['wpd_post_type'])) {

			if ($_POST['wpd_post_type'] == 'post') {
				$this->post_type = array('post');
			}

			if ($_POST['wpd_post_type'] == 'page') {
				$this->post_type = array('page');
			}

			if ($_POST['wpd_post_type'] == 'post,page') {
				$this->post_type = array('post', 'page');
			}

		}

		if (isset($_POST['disbale_modified_days']) && is_numeric($_POST['disbale_modified_days'])) {
			$this->disbale_modified_days = (int) $_POST['disbale_modified_days'];
		}

		$total_diff_in_days = $this->latest_day + $this->earliest_day;

		echo "<div class='wrap'>";
		echo "<h1>Reordering</h1>";

		if ($this->earliest_day>0) {
			echo "<p>The earliest day is {$this->earliest_day} days ago.</p>";
		}

		if ($this->latest_day>0) {
			echo "<p>The latest day is {$this->latest_day} days in the future.</p>";
		}


		$allposts = $this->get_all_posts();
		if (count($allposts) > 0) {

			$doneDays = array();

			echo "<table class='wp-list-table widefat fixed striped posts'>";
			echo "<thead>";
			echo "<tr>";
			echo "<th>Title</th>";
			echo "<th>Current date</th>";
			echo "<th>New publish date</th>";
			echo "<th>New modified date</th>";
			echo "</tr>";
			echo "</thead>";
			foreach($allposts as $id => $postData) {

				$link = $postData['link'];
				$title = $postData['title'];




				$random_days_behind = rand(0,$total_diff_in_days);


				$random_days_behind = (int) $random_days_behind;

				$day_diff = $random_days_behind - ($this->earliest_day );

				if ($day_diff > 0) {

					$plus_minus = '+';

				}
				else {


					$day_diff = abs($day_diff); // get positive
					$plus_minus = '-';


				}

				if ($day_diff == 0) {
					$plus_minus = '';
				}

				// Fixed the issue with undefined value
				if (!isset($doneDays[str_replace("+","",$plus_minus) . $day_diff])) {
					$doneDays[str_replace("+","",$plus_minus) . $day_diff] = 1;
				} else {
					$doneDays[str_replace("+","",$plus_minus) . $day_diff]++;
				}

				$strtotimestring = $plus_minus . $day_diff . ' days';

				$date_time = (strtotime($strtotimestring)); // H:i:s

				// get the year/mo/day from strtotime(- xx days)
				$newDate = date('Y-m-d', $date_time) . " " . $this->getRandomTime();

				if ($this->days_offset <= 0 && $day_diff == 0) {
					$newDate = date('Y-m-d', $date_time) . " " . $this->getRandomTime(true);
				}

				// By default modiefied date is the same as new published date
				$modifiedDate = $newDate;

				// Adding random modified date only for earliest dates
				if ($this->disbale_modified_days == 0) {

					$rand_days = rand(0, $this->earliest_modified_day);

					if ($this->days_offset < 0) {
						$strtotimestring_modified = ($day_diff - $rand_days) . ' days';

						if ($this->earliest_modified_day == 0) {
							$strtotimestring_modified = "0 days";
						}
					}

					if ($this->days_offset >= 0) {
						$strtotimestring_modified = ($day_diff + $rand_days) . ' days';
					}

					if ($this->days_offset < 0) {
						if (substr($strtotimestring_modified,0,1) == '-') {
							$strtotimestring_modified = "0 days";
						}
					}

					$date_time_modified = (strtotime($strtotimestring_modified)); // H:i:s
					$modifiedDate = date('Y-m-d', $date_time_modified) . " " . $this->getRandomTime();

					if ($this->days_offset <= 0 && strtotime($modifiedDate) > time()) {
						$modifiedDate = date('Y-m-d', strtotime('0 days')) . " " . $this->getRandomTime(true);
					}

				}

				$updateArray = array(
					'ID' => $id,
					'post_date' => $newDate,
					'post_date_gmt' => $newDate,
					'post_modified' => $modifiedDate,
					'post_modified_gmt' => $modifiedDate,
					'edit_date' => true,
				);

				$_SESSION['ppcUpdate'][] = $updateArray;

				//$result = wp_update_post($updateArray);

				echo "<tr>
						<td><a href='$link'>$title</a> (<a href='post.php?post=$id&action=edit'>Edit</a>)</td>
						<td>{$postData['orig_date']}</td>
						<td>$newDate</td>
						<td>$modifiedDate</td>
					</tr>";

			}
			echo "</table>";

			echo "<h2>Counters</h2>";

			ksort($doneDays);

			echo "<p>This shows the frequency of each post being posted x days ago/in future. It doesn't show every single day - only the ones with at least one post</p>";
			//change for for() to show all days? might be handy


			echo "<table><thead><th>+/- days</th><th>Number of posts</th></thead>";
			foreach($doneDays as $id => $val) {


				if ($id == -0) {
					$id = "0";
				}

				if ($id > 0) {
					$id = "+" . $id;
				}

				echo "<tr><td>$id days</td><td>$val posts</td></tr>";
			}

			echo "</table>";

			echo "<h2>Confirm update</h2>";
			echo "<p>If you are happy with the new dates, click Update to save changes to database.</p>";

			echo "<form action='".$_SERVER['REQUEST_URI']."' method='post'>";
			echo "<input type='hidden' name='ppc-action' value='write'>";
			echo "<p><input type=\"submit\" class=\"button button-primary\" name=\"completeAction\" value='Update' /></p>";
			echo "</form>";

			echo "</div>";


		}
		else {

			echo "<p>Error - Could not find any posts to reorder!</p>";

		}

	}

	public function update_posts() {

		$updateArray = $_SESSION['ppcUpdate'];
		if (is_array($updateArray)) {
			foreach($updateArray as $update) wp_update_post($update);
			echo "<div class='wrap'>";
			echo "<h1>" . $this->plugin_title . "</h1>";
			echo "<p>Changes are saved to the database.";
			echo "</div>";
		}
		unset($_SESSION['ppcUpdate']);
	}


	protected function getRandomTime($belowNow = false) {
		$newTime = str_pad(rand(0,23), 2, '0', STR_PAD_LEFT) . ":" . str_pad(rand(0,59), 2, '0', STR_PAD_LEFT) . ":" . str_pad(rand(0,59), 2, '0', STR_PAD_LEFT);

		if ($belowNow) {
			$newTime = str_pad(rand(0,date('H')), 2, '0', STR_PAD_LEFT) . ":" . str_pad(rand(0,date('i')), 2, '0', STR_PAD_LEFT) . ":" . str_pad(rand(0,date('s')), 2, '0', STR_PAD_LEFT);
		}

		return $newTime;
	}

	protected function get_all_cats_options() {



		 $args = array(
			'type'                     => array('post', 'page'),
			'child_of'                 => 0,
			'parent'                   => '',
			'orderby'                  => 'name',
			'order'                    => 'ASC',
			'hide_empty'               => 1,
			'hierarchical'             => 1,
			'exclude'                  => '',
			'include'                  => '',
			'number'                   => '',
			'taxonomy'                 => 'category',
			'pad_counts'               => true );


		 $categories = get_categories( $args );

		 $returnhtml = '';
		 foreach($categories as $cat) {

			 $id = $cat->cat_ID;
			 $name = $cat->name;
			 $c = $cat->category_count;
			 $returnhtml .= "<option value='$id'>$name ($c)</option>";

		 }

		 return $returnhtml;

				


	}

		protected function get_all_tags_options() {




			$tags = get_tags();
			$returnhtml = '';
			if ($tags) {
			foreach ($tags as $tag) {

				$id = $tag->term_id;
				$name = $tag->name;
				$c = $tag->count;
				$returnhtml .= "<option value='$id'>$name ($c)</option>";

			}


			}
			else {

				$returnhtml = "<option value=''>(you have no tags!)</option>";

			}

			
			return $returnhtml;

		}

	public function show_options() {


?>

<div class="wrap">
<h1><?php echo $this->plugin_title; ?></h1>

<p class="description">Use this form to set posts/pages in the selected category to a random date.</p>

<form method='post' action='<?=$_SERVER['REQUEST_URI'];?>'>

<input type="hidden" name="tagid" value="">
<input type="hidden" name="latest_day" value="0" />

<table class="form-table">
	<tbody>
	<tr>
		<th scope="row"><label for="incategory">Categories</label></th>
		<td>

			<select name='incategory'>
				<option value=''>All</option>
				<?php
				echo $this->get_all_cats_options();
				?>
			</select>
			<p class="description">An option to select category.</p>
		</td>
	</tr>
	<tr>
		<th scope="row"><label for="wpd_post_type">Post types</label></th>
		<td>
			<fieldset>
			<label><input type="radio" name="wpd_post_type" value="post"> Posts</label><br>
			<label><input type="radio" name="wpd_post_type" value="page"> Pages</label><br>
			<label><input type="radio" name="wpd_post_type" value="post,page" checked="checked"> Posts and Pages</label>
			<p class="description">Select post types to update date values?</p>
			</fieldset>
		</td>
	</tr>
	<tr>
		<th scope="row"><label for="earliest_day">Publish Days</label></th>
		<td>
			<input type="number" id="earliest_day" name="earliest_day" value="-30" />
			<p class="description">Select days offset. 0 is today, negative is for previous days, positive for future days (scheduled posts).</p>
		</td>
	</tr>
	<tr>
		<th scope="row"><label for="earliest_modified_day">Modified Days</label></th>
		<td>
			<div id="earliest_modified_day_tr">
				<input type="number" id="earliest_modified_day" name="earliest_modified_day" value="5"  min="0" />
				<p class="description">Select offset days for modified date according to publish date. 0 is today for modified date.</p>
			</div>
			<div style="margin-top: 10px;">
				<label><input type="checkbox" id="disbale_modified_days" name="disbale_modified_days" value="1" /> Make modified dates the same as the publish dates. </label>
			</div>
		</td>
	</tr>
	</tbody>
</table>

<p class="submit">
	<input type="submit" class="button button-primary" name="random_post_dater" value='Apply' />
</p>

</form>
<script>
	jQuery( document ).ready(function($) {

		$('#disbale_modified_days').click( function() {
			if ($(this).attr('checked')) {
				$('#earliest_modified_day_tr').hide();
			} else {
				$('#earliest_modified_day_tr').show();
			}
		});

		//$('#disbale_modified_days').trigger('click');

	});
</script>
</div>
<?php

	}

}

add_action('admin_menu','ji_wp_post_dater_admin');

function ji_wp_post_dater_admin() {

	add_submenu_page(
		'tools.php',
		__('Posts & Pages Changer'),
		__('Posts & Pages Changer'),
		'manage_options',
		'ji-wp-post-dater',
		'ji_wp_post_dater_page'
	);

}

function ji_wp_post_dater_page() {

	$random_post_dater = new ji_wp_post_dater();

	if (isset($_POST['random_post_dater'] )) {
		$random_post_dater->redate_posts();
	}
	elseif(isset($_POST['ppc-action']) && $_POST['ppc-action']=='write') {
		$random_post_dater->update_posts();
	}
	else {
		$random_post_dater->show_options();
	}

}

add_action('init', 'myStartSession', 1);

function myStartSession() {
	if(!session_id()) {
		session_start();
	}
}



